// BEAction.h: interface for the CBEAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// Since much of C(Better)EditView is concerned with typing, all
// all actions have members DoTyping() and CanStillType(), though
// they may be empty. This simplifies recording a sequence of
// changes and treating it as a whole, as an edit view can keep a
// pointer to a CBEAction object and query it to see if it can
// accomodate more text, all without having to dynamically cast it.
//
//////////////////////////////////////////////////////////////////////

#if !defined(BEACTION_H)
#define BEACTION_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Action.h"

class CBetterEditView;
class CBEAction : public CAction  
{
public:
	CBEAction(CBetterEditView *aView, UINT actionType);
	virtual ~CBEAction();

	virtual void		Undo();
	virtual void		Redo();

	// Since much of Rich(er)EditView is concerned with typing,
	// all doable actions have members DoTyping() and CanStillType(),
	// though they may be empty.
	virtual	void		DoTyping(UINT nChar, UINT nRepCnt, UINT nFlags);
	virtual	bool		CanStillType() const;

						// target of the operation which
						// the action encapsulates
	CBetterEditView		*operationTarget;
};

#endif

