// BEReplaceAction.cpp: implementation of the CBEReplaceAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BEReplaceAction.h"
#include "BetterEditView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBEReplaceAction::CBEReplaceAction(CBetterEditView *aView, UINT actionType, void *d)
	: CBEEditAction(aView, actionType, 0), data(*(ReplaceData *)d)
{
	
}

CBEReplaceAction::~CBEReplaceAction()
{

}

void CBEReplaceAction::PerformOperation()
{
	if (actionType == IDS_REPLACE_ACTION)
	{
		operationTarget->GetEditCtrl().ReplaceSel(data.replaceText);
	}
	else if (actionType == IDS_REPLACEALL_ACTION)
	{
		int start, end;
		do 
		{
			// The first time through we need to save what was replaced

			if (neverUndone)
			{
				CTextRange deleted;

				operationTarget->GetSelection(&start, &end);
				deleted.start = start;
				deleted.end = end;

				// replaced elements are appended from top to bottom, i.e.,
				// the first replaced element is at the front of the vector
				SaveRange(deleted);
				replacedInfo.push_front(deleted);
			}

			operationTarget->GetEditCtrl().ReplaceSel(data.replaceText);

		} while (operationTarget->FindText(data.findText, 1, data.caseSpecific));

		neverUndone = false;
	}
}

void CBEReplaceAction::Undo()
{
	if (actionType == IDS_REPLACE_ACTION)
	{
		CBEEditAction::Undo();
	}
	else if (actionType == IDS_REPLACEALL_ACTION)
	{
		const size_t len = data.replaceText.GetLength();	
		CTextRange deleteRange;

		int start, end;

		operationTarget->GetSelection(&start, &end);
		operationTarget->InhibitRedraw(true);

		typedef ReplacedInfo::const_iterator ci;
		for (ci ptr = replacedInfo.begin(); ptr != replacedInfo.end(); ++ptr)
		{
			// The endpoints of the "bottom" elements are valid only
			// when the items at the top of the document have been
			// replaced. So when we undo, we need to go bottom up, and
			// with the STL we can easily iterate backwards.

			deleteRange.start = (*ptr).start;
			deleteRange.end = deleteRange.start + len;

			DeleteRange(deleteRange);

			RestoreRange((*ptr));
		}

		operationTarget->SetSelection(start, end);
		operationTarget->InhibitRedraw(false);
	}
}

void CBEReplaceAction::Redo()
{
	if (actionType == IDS_REPLACE_ACTION)
	{
		CBEEditAction::Redo();
	}
	else if (actionType == IDS_REPLACEALL_ACTION)
	{
		int start, end;

		operationTarget->GetSelection(&start, &end);
		operationTarget->InhibitRedraw(true);

		// Move to the top of the document and find
		// the initial word; it must exist or we would
		// not be here!

		operationTarget->SetSelection(0, 0);
		operationTarget->FindText(data.findText, 1, data.caseSpecific);

		// It's easiest just to reperform the replace operation
		// than to save all the replacements, clear them, and
		// finally restore the originals.
		PerformOperation();

		operationTarget->SetSelection(start, end);
		operationTarget->InhibitRedraw(false);
	}
}
