// BETextAction.cpp: implementation of the CBETextAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BETextAction.h"
#include "BetterEditView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBETextAction::CBETextAction(CBetterEditView *aView, UINT type)
	: CBEAction(aView, type)
{
	int start, end;

	// save current selection points
	operationTarget->GetSelection(&start, &end);
	deleted.start = inserted.start = start;
	deleted.end = inserted.end = end;

	// we have not undone this action, so we will need to save
	// the "after" state before reverting to a previous state
	neverUndone = true;

	SaveRange(deleted);
}

CBETextAction::~CBETextAction()
{

}

void CBETextAction::Undo()
{
	CBEAction::Undo();

	// save only during the first undo; after
	// that, we have the information we need.
	if (neverUndone)
	{
		neverUndone = false;
		SaveRange(inserted);
	}

	// deleted any inserted text
	DeleteRange(inserted);

	// insert any deleted text, restore selection
	RestoreRange(deleted);
}

void CBETextAction::Redo()
{
	CBEAction::Redo();

	// re-delete the deleted text
	DeleteRange(deleted);

	// re-insert the inserted text
	RestoreRange(inserted);
}

void CBETextAction::SaveRange(CTextRange &aRange)
{
	int start, end;

	// prevent redrawing -- otherwise the repeated
	// process of selecting text causes flashing
	BOOL locked = operationTarget->LockWindowUpdate();
	operationTarget->InhibitRedraw(true);

	operationTarget->GetSelection(&start, &end);

	operationTarget->SetSelection(aRange.start, aRange.end);
	aRange.text = operationTarget->GetTextSelection();

	operationTarget->SetSelection(start, end);
	operationTarget->InhibitRedraw(false);
	if (locked)
		operationTarget->UnlockWindowUpdate();
}

void CBETextAction::DeleteRange(const CTextRange &aRange)
{
	operationTarget->SetSelection(aRange.start, aRange.end);
	operationTarget->PerformClear();
}

void CBETextAction::RestoreRange(const CTextRange &aRange)
{
	BOOL locked = operationTarget->LockWindowUpdate();
	operationTarget->InhibitRedraw(true);

	operationTarget->SetSelection(aRange.start, aRange.start);
	operationTarget->InsertText(aRange.text);
	operationTarget->SetSelection(aRange.start, aRange.end);

	operationTarget->InhibitRedraw(false);
	if (locked)
		operationTarget->UnlockWindowUpdate();
}

void CBETextAction::Finish()
{
	CBEAction::Finish();

	int start, end;

	// save insertion endpoint
	operationTarget->GetSelection(&start, &end);
	inserted.end = end;
}
