// BETypingAction.h: interface for the CBETypingAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// This class records characters which have been typed and/or
// deleted. It separates normal characters from backspace and
// forward delete, and has the only non-trivial implementations
// of DoTyping() and CanStillType().
//
// This class is based on classes in the THINK Class Library
// and the MacApp application frameworks.
//
//////////////////////////////////////////////////////////////////////

#if !defined(BETYPINGACTION_H)
#define BETYPINGACTION_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "BETextAction.h"

class CBETypingAction : public CBETextAction  
{
public:
						CBETypingAction(CBetterEditView *aView, UINT actionType, void *data);
	virtual				~CBETypingAction();

						// The real "Do()" of this class.
	virtual	void		DoTyping(UINT nChar, UINT nRepCnt, UINT nFlags) ;

						// Indicates whether this action can accommodate more typing
	virtual	bool		CanStillType() const;

						// Disallows further typing in this action. A new action
						// will be created for subsequent characters.
	virtual void		Finish();

						// Do NOT call; unlike other actions, the typing action
						// performs its operation through calls to DoTyping().
	virtual	void		Do();

protected:
	virtual void		DoNormalChar	(UINT nChar, UINT nRepCnt, UINT nFlags);
	virtual void		DoBackspace		(UINT nChar, UINT nRepCnt, UINT nFlags);
	virtual void		DoFwdDelete		(UINT nChar, UINT nRepCnt, UINT nFlags);


						// Indicates whether this is the first character which
						// has been typed; used to notify undo manager only once.
	bool				firstChar;

						// flag indicating whether typing is still possible with
						// with this typing action or whether a new one must be
						// be created; e.g., can't continue typing after we've
						// undone the typing.
	bool				stillTyping;
};

#endif
