// BetterEditView.cpp : implementation file
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//

#include "stdafx.h"
#include "BetterEditView.h"
#include "UMResources.h"
#include "UndoManager.h"
#include "SimpleUndoRedoMenu.h"
#include "BEEditAction.h"
#include "BETypingAction.h"
#include "BEReplaceAction.h"

#include <assert.h>
#include <afxodlgs.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CFindReplaceDialog *CBetterEditView::findReplaceDialog = 0;
static const UINT nMsgFindReplace = ::RegisterWindowMessage(FINDMSGSTRING);

/////////////////////////////////////////////////////////////////////////////
// CBetterEditView

IMPLEMENT_DYNCREATE(CBetterEditView, CEditView)

CBetterEditView::CBetterEditView()
	: lastAction(0), redraw(true), locked(FALSE), actionTarget(0),
	  lastChar(VK_NONAME)
{
}

int CBetterEditView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (GetActionTarget() == 0)
		SetActionTarget(GetDocument()); 
	
	return 0;
}

BEGIN_MESSAGE_MAP(CBetterEditView, CEditView)
	//{{AFX_MSG_MAP(CBetterEditView)
	ON_WM_CHAR()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_EDIT_CLEAR, OnEditClear)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_UNDO_0, ID_UNDO_9, OnMultipleUndo)
	ON_UPDATE_COMMAND_UI_RANGE(ID_UNDO_0, ID_UNDO_9, OnUpdateMultipleUndo)
	ON_COMMAND_RANGE(ID_REDO_0, ID_REDO_9, OnMultipleRedo)
	ON_UPDATE_COMMAND_UI_RANGE(ID_REDO_0, ID_REDO_9, OnUpdateMultipleRedo)
	ON_REGISTERED_MESSAGE(nMsgFindReplace, OnFindReplaceCmd)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBetterEditView diagnostics

#ifdef _DEBUG
void CBetterEditView::AssertValid() const
{
	CEditView::AssertValid();
}

void CBetterEditView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBetterEditView message handlers

// The four members eliminate the need for a deriving a new document class
// just to handle undo/redo. Note the document object is still used to record
// actions with the Undo Manager, as a single document may have several views,
// each just a "window" on the real data. This is also typical of how the
// MFC Library already handles undo/redo. Unfortunately, you will need to
// derive a subclass if the document is not the correct action target.

void CBetterEditView::OnEditUndo() 
{
	// tell undo manager to undo the last action
	// associate with the document (action target)
	gUndoManager->Undo(GetActionTarget());
}

void CBetterEditView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
//	was: CSimpleUndoRedoMenu::Set(pCmdUI, GetActionTarget());
	if (gUndoManager->CanUndo(GetActionTarget()))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);

}

void CBetterEditView::OnEditRedo() 
{
	gUndoManager->Redo(GetActionTarget());
}

void CBetterEditView::OnUpdateEditRedo(CCmdUI* pCmdUI) 
{
//	was: CSimpleUndoRedoMenu::Set(pCmdUI, GetActionTarget());
	if (gUndoManager->CanRedo(GetActionTarget()))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

// New handlers for multiple, simultaneous undo/redo
void CBetterEditView::OnMultipleUndo(UINT nID) 
{
	const long nActions = nID - ID_UNDO_0 + 1L;
	gUndoManager->Undo(GetActionTarget(), nActions);
}

void CBetterEditView::OnUpdateMultipleUndo(CCmdUI* pCmdUI) 
{
	const long nActions = pCmdUI->m_nID - ID_UNDO_0 + 1L;
	if (gUndoManager->CanUndo(GetActionTarget(), nActions))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CBetterEditView::OnMultipleRedo(UINT nID) 
{
	const long nActions = nID - ID_REDO_0 + 1L;
	gUndoManager->Redo(GetActionTarget(), nActions);
}

void CBetterEditView::OnUpdateMultipleRedo(CCmdUI* pCmdUI) 
{
	const long nActions = pCmdUI->m_nID - ID_REDO_0 + 1L;
	if (gUndoManager->CanRedo(GetActionTarget(), nActions))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

// Start of the standard message map overrides. Most of these following the
// following pattern, where XXX is the operation/command name:

//		void CBetterEditView::PerformXXX()
//				-- calls base handler, usually CBetterEditView::OnXXX()

//		void CBetterEditView::OnXXX()
//				-- creates action object, then invokes Do(); action
//				-- objects eventually calls XXXOperation() after recording
//				-- appropriate state of view/document object

void CBetterEditView::PerformClear()
{
	CEditView::OnEditClear();
}

void CBetterEditView::OnEditClear() 
{
	lastAction = MakeEditAction(IDS_CLEAR_ACTION);
	lastAction->Do();
}

void CBetterEditView::PerformCut() 
{
	CEditView::OnEditCut();
}

void CBetterEditView::OnEditCut() 
{
	lastAction = MakeEditAction(IDS_CUT_ACTION);
	lastAction->Do();	
}

void CBetterEditView::PerformPaste() 
{
	CEditView::OnEditPaste();
}

void CBetterEditView::OnEditPaste() 
{
	lastAction = MakeEditAction(IDS_PASTE_ACTION);
	lastAction->Do();
}

void CBetterEditView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Different characters are handled with different messages,
	// e.g., delete occurs during WM_KEYDOWN while normals chars
	// are handled during WM_CHAR.

	// Worse, the Edit & Rich Edit controls seem to differ in how
	// they handle chars too. E.g., backspace is handled during a
	// WM_CHAR in Edit control whereas it's handled during WM_KEYDOWN
	// in the Rich Edit control.

	// Through testing, I believe this routine handles all normal
	// characters plus the backspace key, ie, no delete, arrows, etc.

	if (!IsExtendedKey(nFlags))
	{
		// We don't do anything if the user is backspacing at
		// the beginning of the text; there's nothing to undo
		if (nChar == VK_BACK)
		{
			int start, end;

			GetSelection(&start, &end);
			if (start == end && start == 0)
			{
				CEditView::OnChar(nChar, nRepCnt, nFlags);
				return;
			}
		}

		// Okay, there's something that will be deleted; now see 
		// if we need to create a new action or whether we can
		// reuse the last one. We do this when

		//		* there wasn't a last action, typing or otherwise
		//		* we've just started to backspace over previous text
		//		* or for some reason, the last action can't accommodate
		//		  more typing (may not even be a typing action)
		if (
				lastAction == 0
			||
				(nChar == VK_BACK && lastChar != VK_BACK)
			||
				lastAction->CanStillType() == false
			)
		{
			lastAction = MakeTypingAction(IDS_TYPING_ACTION);
		}

		// finally do the typing
		lastAction->DoTyping(nChar, nRepCnt, nFlags);
	}
	else
		// either an action was created below or the char doesn't
		// need an action (it may be something like shift or ctrl)
		CEditView::OnChar(nChar, nRepCnt, nFlags);
}

void CBetterEditView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Different characters are handled with different messages,
	// e.g., delete occurs during WM_KEYDOWN while normals chars
	// are handled during WM_CHAR.

	// Worse, the Edit & Rich Edit controls seem to differ in how
	// they handle chars too. E.g., backspace is handled during a
	// WM_CHAR in Edit control whereas it's handled during WM_KEYDOWN
	// in the Rich Edit control.

	// Through testing, I believe this routine handles only the
	// (forward) delete key. All others are passed through.

	if (nChar == VK_DELETE)
	{
		int start, end;

		// We don't do anything if the user is forward deleting at
		// the very end of the text; there isn't anything to undo
		GetSelection(&start, &end);
		if (start == end && start == GetLength())
		{
			CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
			return;
		}

		// Okay, there's something that will be deleted; now see 
		// if we should create a new action or whether we can re-
		// use the last one. We create a new action when

		//		* there wasn't a last action, typing or otherwise
		//		* or for some reason, the last action can't accommodate
		//		  more typing (may not even be a typing action)


		if (lastAction == 0 || lastAction->CanStillType() == false)
		{
			lastAction = MakeTypingAction(IDS_TYPING_ACTION);
		}

		// finally do the typing
		lastAction->DoTyping(nChar, nRepCnt, nFlags);
	}
	else if (IsCursorKey(nChar))
	{
		// cursor keys don't affect the text, just the cursor
		SelectionChanged();
		CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	}
	else
		// the character is handled in OnChar()
		CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CBetterEditView::PerformTyping(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// This routine is the bottleneck for typing, ie, after this
	// routine executes, a new character has been applied to the
	// buffer. Note that a delete or backspace can actually
	// remove characters from the it, so it does not always add.

	// Because different keys are handled at different times, the
	// base member which is called depends on the character.

	if (nChar == VK_DELETE)
	{
		CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	}
	else
	{
		CEditView::OnChar(nChar, nRepCnt, nFlags);
	}

	lastChar = nChar;
}

LRESULT CBetterEditView::OnFindReplaceCmd(WPARAM wParam, LPARAM lParam)
{
	// We can only hope this is the right one, as the edit state
	// classes are "private" to MFC (in afximpl.h).
	findReplaceDialog = CFindReplaceDialog::GetNotifier(lParam);
	ASSERT(findReplaceDialog != NULL);

	return CEditView::OnFindReplaceCmd(wParam, lParam);
}

void CBetterEditView::OnReplaceSel(LPCTSTR lpszFind, BOOL bNext, BOOL bCase, LPCTSTR lpszReplace)
{
	ASSERT_VALID(this);
	ASSERT(findReplaceDialog != NULL);

	if (!InitializeReplace(lpszFind, bNext, bCase))
		return;


	lastAction = MakeReplaceAction(IDS_REPLACE_ACTION, (void *)(LPCTSTR)lpszReplace);
	lastAction->Do();

	FindText(lpszFind, bNext, bCase);

	ASSERT_VALID(this);
}

void CBetterEditView::OnReplaceAll(LPCTSTR lpszFind, LPCTSTR lpszReplace, BOOL bCase)
{
	ASSERT_VALID(this);
	ASSERT(findReplaceDialog != NULL);

	CWaitCursor wait;
	// no selection or different than what looking for
	if (!InitializeReplace(lpszFind, TRUE, bCase) &&
		!SameAsSelected(lpszFind, bCase))
	{
		// initial find was not successful
		return;
	}

	ReplaceData data(lpszReplace, lpszFind, bCase);
	lastAction = MakeReplaceAction(IDS_REPLACEALL_ACTION, &data);
	lastAction->Do();

	ASSERT_VALID(this);
}

BOOL CBetterEditView::InitializeReplace(LPCTSTR lpszFind, BOOL bNext, BOOL bCase)
	// helper to do find first if no selection
{
	// do find next if no selection
	int nStartChar, nEndChar;
	GetEditCtrl().GetSel(nStartChar, nEndChar);
	if (nStartChar == nEndChar)
	{
		if (!FindText(lpszFind, bNext, bCase))
		{
			// text not found
			OnTextNotFound(lpszFind);
		}
		return FALSE;
	}

	if (!SameAsSelected(lpszFind, bCase))
	{
		if (!FindText(lpszFind, bNext, bCase))
		{
			// text not found
			OnTextNotFound(lpszFind);
		}
		return FALSE;
	}

	ASSERT_VALID(this);
	return TRUE;
}


CBEAction *CBetterEditView::MakeEditAction(UINT actionType, void *data)
{
	return new CBEEditAction(this, actionType, data);
}

CBEAction *CBetterEditView::MakeTypingAction(UINT actionType, void *data)
{
	return new CBETypingAction(this, actionType, data);
}

CBEAction *CBetterEditView::MakeReplaceAction(UINT actionType, void *data)
{
	return new CBEReplaceAction(this, actionType, data);
}

void CBetterEditView::SelectionChanged()
{
	if (lastAction)
	{
		if (lastAction->IsFinished() == false)
			lastAction->Finish();
		lastAction = 0;
	}
}

void CBetterEditView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SelectionChanged();
	CEditView::OnLButtonDown(nFlags, point);
}

void CBetterEditView::GetItemBefore(int &pos, CTextRange &aRange)
{
	const int len = GetLength();
	if (pos != 0)
	{
		int selStart, selEnd;

		// save current selection
		GetSelection(&selStart, &selEnd);

		// set to selection to previous char,
		// noting that the end of line consists
		// of two characters, "\r\n", which can
		// be selected individually
		int start, end;
		SetSelection(pos - 1, pos);

		GetSelection(&start, &end);
		aRange.start = start;
		aRange.end = end;
		aRange.text = GetTextSelection();

		SetSelection(selStart, selEnd);
	}
	else
	{
		aRange.end = aRange.start = 0;
	}
}

void CBetterEditView::GetItemAfter(int &pos, CTextRange &aRange)
{
	const int len = GetLength();
	assert(0 <= pos && pos <= len);

	if (pos < len)
	{
		int selStart, selEnd;

		GetSelection(&selStart, &selEnd);
		SetSelection(pos, pos + 1);

		// test for funny end of line
		int testStart, testEnd;
		GetSelection(&testStart, &testEnd);
		if (testStart == testEnd)
		{
			SetSelection(pos, pos + 2);
		}

		int start, end;

		// handle long vs. int issue
		GetSelection(&start, &end);
		aRange.start = start;
		aRange.end = end;

		aRange.text = GetTextSelection();

		SetSelection(selStart, selEnd);
	}
	else
	{
		aRange.end = aRange.start = pos;
	}

}

void CBetterEditView::InhibitRedraw(const bool disable)
{
	if (disable)
	{
		if (redraw == true)
		{
			redraw = false;
			locked = LockWindowUpdate();
		}
	}
	else
	{
		if (redraw == false)
		{

			redraw = true;
			if (locked)
			{
				locked = false;
				UnlockWindowUpdate();
			}
		}
	}
}

void CBetterEditView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	// force menu rebuild by creating "NULL" action
	// (eventually, the notification callback calls
	// the appropriate menu building routine)
	if (bActivate && IsWindowVisible())
		gUndoManager->SetLastAction(GetActionTarget(), 0);
	CEditView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

