// MultipleUndoRedoMenu.cpp: implementation of the CMultipleUndoRedoMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MultipleUndoRedoMenu.h"
#include "UndoManager.h"
#include "UMResources.h"

#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const CString CMultipleUndoRedoMenu::undoItem( _T("&Undo") );
const CString CMultipleUndoRedoMenu::redoItem( _T("&Redo") );

void CMultipleUndoRedoMenu::Get(CMenu *menuBar, CMenu *&undoMenu, CMenu *&redoMenu)
{		
	CMenu *editMenu = 0;
	CString anItem;
	const int numMenus = menuBar->GetMenuItemCount();

	// assumes undo/redo in Edit menu
	for (int i = 0; i < numMenus; ++i)
	{
		CMenu *aMenu = menuBar->GetSubMenu(i);
		if (aMenu)
		{
			aMenu->GetMenuString(0, anItem, MF_BYPOSITION);
			if (anItem.Left(5) == undoItem)
			{
				editMenu = aMenu;
				break;
			}
		}
	}

	// it has to be there
	if (editMenu)
	{
		// see if we've already added the undo submenu
		undoMenu = editMenu->GetSubMenu(0);
		if (undoMenu == 0)
		{
			// nope, create an empty submenu and
			// attach it to the undo menu item
			CMenu subMenu;

			subMenu.CreatePopupMenu();
			editMenu->ModifyMenu(0, MF_BYPOSITION | MF_POPUP, (UINT)subMenu.Detach(), undoItem);
			undoMenu = editMenu->GetSubMenu(0);

			assert(undoMenu);
		}

		// see if we've already added the redo submenu
		redoMenu = editMenu->GetSubMenu(1);
		if (redoMenu == 0)
		{
			// nope, create an empty submenu and
			// attach it to the undo menu item
			CMenu subMenu;

			subMenu.CreatePopupMenu();
			editMenu->ModifyMenu(1, MF_BYPOSITION | MF_POPUP, (UINT)subMenu.Detach(), redoItem);
			redoMenu = editMenu->GetSubMenu(1);

			assert(redoMenu);
		}
	}
}

void CMultipleUndoRedoMenu::Build(CMenu *menuBar, const void *actionTarget)
{
	if (menuBar)
	{
		const long menuCapacity = LAST_UNDO_ID - ID_UNDO_0 + 1L;
		CMenu *menu[2];

		// find (and construct if necessary) the undo and redo submenus
		Get(menuBar, menu[CUndoManager::StackKind::UNDO], menu[CUndoManager::StackKind::REDO]);

		// then build each one from scratch (fairly tricky to just
		// accommodate the changes made by the undo/redo operation)
		for(int i = 0; i < 2; ++i)
		{
			int j;
			const CUndoManager::StackKind k = (CUndoManager::StackKind)i;
			const UINT menuSize = menu[k]->GetMenuItemCount();

			// clear out what was there
			for(j = 0; j < menuSize; ++j)
				menu[k]->RemoveMenu(0, MF_BYPOSITION);

			// the menus and their associated stacks may be of different
			// sizes (either could be larger than the other); note that
			// the redo menu is bounded by the undo menu, just as the redo
			// stack is bounded by the undo stack.
			const long stackSize = gUndoManager->GetStackSize(actionTarget, k);
			const long menuLimit = (stackSize < menuCapacity) ? stackSize : menuCapacity;
			const UINT id_base = ID_UNDO_0 + (k * menuCapacity);

			// add in the new action labels
			for(j = 0; j < menuLimit; ++j)
			{
				const CString &actionName = gUndoManager->GetActionName(actionTarget, k, j);
				menu[i]->AppendMenu(MF_STRING, id_base + j, actionName);
			}
		}
	}
}

void UMSubMenuBuilder(void *actionTarget, UMNotificationData &data)
{
	// assumes the main wnd is associated with the action target
	// (not necessarily true but probably a reasonable assumption)
	CMultipleUndoRedoMenu::Build(AfxGetMainWnd()->GetMenu(), actionTarget);
	UMCmdTargetNotifier(actionTarget, data);
}