// ParagraphStyle.h: interface for the CParagraphStyle class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// Utility data structure for storing a single paragraph style.
//
//////////////////////////////////////////////////////////////////////

#if !defined(PARAGRAPHSTYLE_H)
#define PARAGRAPHSTYLE_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <richedit.h>

class CParagraphStyle  
{
public:
	CParagraphStyle()
	{
		count = 0;

		format.cbSize = sizeof(PARAFORMAT);
		format.dwMask = 0;
		format.wNumbering = 0;
		format.dxStartIndent = 0;
		format.dxRightIndent = 0;
		format.dxOffset = 0;
		format.wAlignment = 0;
		format.cTabCount = 0;
		// rgxTabs is controlled
		// above member variable
	}

	CParagraphStyle(const PARAFORMAT &pf)
	{
		count = 1;

		format.cbSize = sizeof(PARAFORMAT);
		format.dwMask = pf.dwMask;
		format.wNumbering = pf.wNumbering;
		format.dxStartIndent = pf.dxStartIndent;
		format.dxRightIndent = pf.dxRightIndent;
		format.dxOffset = pf.dxOffset;
		format.wAlignment = pf.wAlignment;
		format.cTabCount = pf.cTabCount;
		for(short i = 0; i < format.cTabCount; ++i)
			format.rgxTabs[i] = pf.rgxTabs[i];
	}

	CParagraphStyle(const CParagraphStyle &ps)
	{
		count = ps.count;

		format.cbSize = sizeof(PARAFORMAT);
		format.dwMask = ps.format.dwMask;
		format.wNumbering = ps.format.wNumbering;
		format.dxStartIndent = ps.format.dxStartIndent;
		format.dxRightIndent = ps.format.dxRightIndent;
		format.dxOffset = ps.format.dxOffset;
		format.wAlignment = ps.format.wAlignment;
		format.cTabCount = ps.format.cTabCount;
		for(short i = 0; i < format.cTabCount; ++i)
			format.rgxTabs[i] = ps.format.rgxTabs[i];
	}

	~CParagraphStyle() { }

	const CParagraphStyle &operator=(const CParagraphStyle &ps)
	{
		if (this != &ps)
		{
			count = ps.count;

			format.cbSize = sizeof(PARAFORMAT);
			format.dwMask = ps.format.dwMask;
			format.wNumbering = ps.format.wNumbering;
			format.dxStartIndent = ps.format.dxStartIndent;
			format.dxRightIndent = ps.format.dxRightIndent;
			format.dxOffset = ps.format.dxOffset;
			format.wAlignment = ps.format.wAlignment;
			format.cTabCount = ps.format.cTabCount;

			for(short i = 0; i < format.cTabCount; ++i)
				format.rgxTabs[i] = ps.format.rgxTabs[i];
		}

		return *this;
	}

	void SetMask(const DWORD dwMask =
								  PFM_ALIGNMENT		| PFM_NUMBERING
								| PFM_OFFSET		| PFM_OFFSETINDENT
								| PFM_RIGHTINDENT	| PFM_STARTINDENT
								| PFM_TABSTOPS)
	{
		format.dwMask = dwMask;
	}

	PARAFORMAT format;
	long count;
};

inline bool operator==(const CParagraphStyle &ps1, const CParagraphStyle &ps2)
{
	// ignore start and end

	if (	ps1.format.dwMask == ps2.format.dwMask
		&&
			ps1.format.wNumbering == ps2.format.wNumbering
		&&
			ps1.format.dxStartIndent == ps2.format.dxStartIndent
		&&
			ps1.format.dxRightIndent == ps2.format.dxRightIndent
		&&
			ps1.format.dxOffset == ps2.format.dxOffset
		&&
			ps1.format.wAlignment == ps2.format.wAlignment
		&&
			ps1.format.cTabCount == ps2.format.cTabCount
		)
		{
			for(short i = 0; i < ps1.format.cTabCount; ++i)
				if (ps1.format.rgxTabs[i] != ps2.format.rgxTabs[i])
					return false;

			return true;
		}
		else
			return false;
}

inline bool operator!=(const CParagraphStyle &ps1, const CParagraphStyle &ps2)
{
	return !(ps1 == ps2);
}

#endif // !defined(AFX_PARAGRAPHSTYLE_H__9D1A1A62_BD97_11D1_8932_444553540000__INCLUDED_)
