// REParagraphAction.cpp: implementation of the CREParagraphAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "REAction.h"
#include "REParagraphAction.h"
#include "RicherEditView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CREParagraphAction::CREParagraphAction(CRicherEditView *aView, UINT type, void *data)
	: CREAction(aView, type), neverUndone(true), alignment(0), newRange(0)
{
	oldRange = operationTarget->MakeTextRange();
	operationTarget->InhibitRedraw(true);
	operationTarget->GetParagraphStyles(*oldRange);
	operationTarget->InhibitRedraw(false);

	switch(actionType)
	{
		case IDS_LEFTJUSTIFICATION_ACTION:
		case IDS_CENTERJUSTIFICATION_ACTION:
		case IDS_RIGHTJUSTIFICATION_ACTION:
			alignment = *(WORD *)data;
			break;
		case IDS_PARAGRAPHFORMATTING_ACTION:
			paraformat = *(PARAFORMAT *)data;
			break;

		case IDS_TABSFORMATTING_ACTION:
			paraformat = *(PARAFORMAT *)data;
			break;
	}
}

CREParagraphAction::~CREParagraphAction()
{
	delete oldRange;
	delete newRange;
}

void CREParagraphAction::Do()
{
	CREAction::Do();

	switch (actionType)
	{
		case IDS_LEFTJUSTIFICATION_ACTION:
		case IDS_CENTERJUSTIFICATION_ACTION:
		case IDS_RIGHTJUSTIFICATION_ACTION:
			operationTarget->PerformParaAlign(alignment);
			break;
		case IDS_BULLETS_ACTION:
			operationTarget->PerformBullet();
			break;
		case IDS_PARAGRAPHFORMATTING_ACTION:
		case IDS_TABSFORMATTING_ACTION:
			operationTarget->SetParaFormat(paraformat);
			break;
	}
}

void CREParagraphAction::Undo()
{
	CREAction::Undo();

	operationTarget->InhibitRedraw(true);
	operationTarget->SetSelection(oldRange->start, oldRange->end);

	if (neverUndone)
	{
		neverUndone = false;
		newRange = operationTarget->MakeTextRange();
		operationTarget->GetParagraphStyles(*newRange);
	}

	operationTarget->SetParagraphStyles(*oldRange);
	operationTarget->InhibitRedraw(false);
}

void CREParagraphAction::Redo()
{
	CREAction::Redo();

	operationTarget->InhibitRedraw(true);
	operationTarget->SetSelection(newRange->start, newRange->end);
	operationTarget->SetParagraphStyles(*newRange);
	operationTarget->InhibitRedraw(false);
}
