// REParagraphAction.h: interface for the CREParagraphAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// This class provides undo-able paragraph formatting operations
// like justification and bullets. Like the CREStyleAction class,
// this class does not need to record text, only (paragraph) style
// information.
//
// This class also does not serve as a base for other subclasses which
// actually style the paragraphs; instead this class handles all such
// operations with a union and the action type to accomodate each op's
// different internal data structures. New operations would however
// require a derived class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(REPARAGRAPHACTION_H)
#define REPARAGRAPHACTION_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "REAction.h"
#include "TextActionUtil.h"

class CREParagraphAction : public CREAction  
{
public:
	CREParagraphAction(CRicherEditView *aView, UINT actionType, void *data = 0);
	virtual ~CREParagraphAction();

	void Do();
	void Undo();
	void Redo();

	CTextRange *oldRange, *newRange;
	bool neverUndone;

	union
	{
		WORD alignment;
		PARAFORMAT paraformat;
	};
};

#endif
