// REReplaceAction.cpp: implementation of the CREReplaceAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "REReplaceAction.h"
#include "RicherEditView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CREReplaceAction::CREReplaceAction(CRicherEditView *aView, UINT actionType, void *d)
	: CREEditAction(aView, actionType, 0), data(*(ReplaceData *)d)
{

}

CREReplaceAction::~CREReplaceAction()
{
	for(ReplacedInfo::iterator ptr = replacedInfo.begin(); ptr != replacedInfo.end(); ++ptr)
	{
		delete (*ptr);
	}
}

void CREReplaceAction::PerformOperation()
{
	if (actionType == IDS_REPLACE_ACTION)
	{
		operationTarget->GetRichEditCtrl().ReplaceSel(data.replaceText);
	}
	else if (actionType == IDS_REPLACEALL_ACTION)
	{
		do 
		{
			// The first time through we need to save what was replaced,
			// including all styling information and derived attributes.
			if (neverUndone)
			{
				CTextRange *deleted = operationTarget->MakeTextRange();

				operationTarget->GetSelection(&(deleted->start), &(deleted->end));
				SaveRange(deleted);

				// replaced elements are appended from top to bottom, i.e.,
				// the first replaced element is at the front of the vector
				replacedInfo.push_back(deleted);
			}

			operationTarget->GetRichEditCtrl().ReplaceSel(data.replaceText);

		} while (operationTarget->FindTextSimple(data.findText,
												 data.caseSpecific,
												 data.wholeWord));

		neverUndone = false;
	}
}

void CREReplaceAction::Undo()
{
	if (actionType == IDS_REPLACE_ACTION)
	{
		CREEditAction::Undo();
	}
	else if (actionType == IDS_REPLACEALL_ACTION)
	{
		const size_t len = data.replaceText.GetLength();	
		CTextRange *deleteRange = operationTarget->MakeTextRange();

		long start, end;

		operationTarget->GetSelection(&start, &end);
		operationTarget->InhibitRedraw(true);

		typedef ReplacedInfo::reverse_iterator ci;
		for (ci ptr = replacedInfo.rbegin(); ptr != replacedInfo.rend(); ++ptr)
		{
			// The endpoints of the "bottom" elements are valid only
			// when the items at the top of the document have been
			// replaced. So when we undo, we need to go bottom up, and
			// with the STL we can easily iterate backwards.

			deleteRange->start = (*ptr)->start;
			deleteRange->end = deleteRange->start + len;

			DeleteRange(deleteRange);

			RestoreRange((*ptr));
		}

		operationTarget->SetSelection(start, end);
		operationTarget->InhibitRedraw(false);
		delete deleteRange;
	}
}

void CREReplaceAction::Redo()
{
	if (actionType == IDS_REPLACE_ACTION)
	{
		CREEditAction::Redo();
	}
	else if (actionType == IDS_REPLACEALL_ACTION)
	{

		long start, end;

		operationTarget->GetSelection(&start, &end);
		operationTarget->InhibitRedraw(true);
		
		// Move to the top of the document and find the
		// initial word; it must exist or we can't be here!
		operationTarget->SetSelection(0, 0);
		operationTarget->FindTextSimple(data.findText,
										data.caseSpecific,
										data.wholeWord);

		// It's easiest just to reperform the replace operation
		// than to save all the replacements, clear them, and
		// finally restore the originals.
		PerformOperation();

		operationTarget->SetSelection(start, end);
		operationTarget->InhibitRedraw(false);
	}
}
