// REStyleAction.cpp: implementation of the CREStyleAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "REStyleAction.h"
#include "RicherEditView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CREStyleAction::CREStyleAction(CRicherEditView *aView, UINT type, void *data)
	: CREAction(aView, type), neverUndone(true), newRange(0)

{
	oldRange = operationTarget->MakeTextRange();
	operationTarget->InhibitRedraw(true);
	operationTarget->GetTextStyles(*oldRange);
	operationTarget->InhibitRedraw(false);

	switch(actionType)
	{
		case IDS_FONTFORMATTING_ACTION:
		case IDS_SIZECHANGE_ACTION:
			format = *(CHARFORMAT *)data;
			if (format.dwMask == (CFM_FACE | CFM_CHARSET))
				name = format.szFaceName;
			break;
		case IDS_BOLD_ACTION:
		case IDS_ITALICS_ACTION:
		case IDS_UNDERLINE_ACTION:
		case IDS_STYLECHANGE_ACTION:
			styleData = *(StyleActionData *)data;
			break;
		case IDS_COLORCHANGE_ACTION:
			nID = *(UINT *)data;
			break;
	}
}

CREStyleAction::~CREStyleAction()
{
	delete oldRange;
	delete newRange;
}

void CREStyleAction::Do()
{
	CREAction::Do();

	switch (actionType)
	{
		case IDS_FONTFORMATTING_ACTION:
		case IDS_SIZECHANGE_ACTION:
			operationTarget->SetCharFormat(format);
			break;
		case IDS_BOLD_ACTION:
		case IDS_ITALICS_ACTION:
		case IDS_UNDERLINE_ACTION:
		case IDS_STYLECHANGE_ACTION:
			operationTarget->PerformStyle(styleData.mask, styleData.effect);
			break;
		case IDS_COLORCHANGE_ACTION:
			operationTarget->PerformColorPick(nID);
			break;
		case IDS_DEFAULTCOLOR_ACTION:
			operationTarget->PerformColorDefault();
			break;
		default:
			// do nothing, subclass can override
			;
	}

}

void CREStyleAction::Undo()
{
	CREAction::Undo();

	operationTarget->InhibitRedraw(true);
	operationTarget->SetSelection(oldRange->start, oldRange->end);

	if (neverUndone)
	{
		neverUndone = false;
		newRange = operationTarget->MakeTextRange();
		operationTarget->GetTextStyles(*newRange);
	}

	operationTarget->SetTextStyles(*oldRange);
	operationTarget->InhibitRedraw(false);
}

void CREStyleAction::Redo()
{
	CREAction::Redo();

	operationTarget->InhibitRedraw(true);
	operationTarget->SetSelection(newRange->start, newRange->end);
	operationTarget->SetTextStyles(*newRange);
	operationTarget->InhibitRedraw(false);
}
