// REStyleAction.h: interface for the CREStyleAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// This class encapsulates styling operations like bold or font
// changes. Because style information is independent of the text
// it styles, this class does not record any text and hence, is
// not a subclass of CRETextAction.
//
// Note that there is quite a variety of styling operations, each
// of which has distinct data structures associated with it; rather
// than have several subclasses, this class uses a union along
// along with the actionType to save the appropriate information and
// perform the correct operation. New operations, such an outline
// style, would require a derived class however.
//
//////////////////////////////////////////////////////////////////////

#if !defined(RESTYLEACTION_H)
#define RESTYLEACTION_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "REAction.h"
#include "RicherEditView.h"

class CREStyleAction : public CREAction  
{
public:
	CREStyleAction(CRicherEditView *aView, UINT actionType, void *data = 0);
	virtual ~CREStyleAction();

	virtual void		Do();
	virtual	void		Undo();
	virtual	void		Redo();

protected:
	CTextRange			*oldRange, *newRange;
	bool				neverUndone;
	union
	{
		CHARFORMAT format;
		StyleActionData styleData;
		UINT nID;
	};
};

#endif
