// RETextAction.cpp: implementation of the CRETextAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RETextAction.h"
#include "RicherEditView.h"

#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRETextAction::CRETextAction(CRicherEditView *aView, UINT type)
	: CREAction(aView, type)
{
	// typing and other text actions may require
	// the inserted action to exist (remove with neverUndone)
	inserted = operationTarget->MakeTextRange();
	deleted  = operationTarget->MakeTextRange();

	// save current selection points
	operationTarget->GetSelection(&deleted->start, &deleted->end);
	inserted->start = deleted->start;
	inserted->end = deleted->end;

	// we haven't undone this action, so we
	// will need to save the "after" state
	// before reverting to a previous state
	neverUndone = true;

	SaveRange(deleted);
}

CRETextAction::~CRETextAction()
{
	delete inserted;
	delete deleted;
}

void CRETextAction::Undo()
{
	CREAction::Undo();

	// save only during the first undo; after
	// that, we have the information we need.
	if (neverUndone)
	{
		neverUndone = false;
		SaveRange(inserted);
	}

	// deleted any inserted text
	DeleteRange(inserted);

	// insert any deleted text, restore selection
	RestoreRange(deleted);
}

void CRETextAction::Redo()
{
	CREAction::Redo();

	// re-delete the deleted text
	DeleteRange(deleted);

	// re-insert the inserted text
	RestoreRange(inserted);
}

void CRETextAction::SaveRange(CTextRange *aRange)
{
	long start, end;

	assert(aRange != 0);

	// prevent redrawing -- otherwise the repeated
	// process of selecting text causes flashing
	const BOOL locked = operationTarget->LockWindowUpdate();
	operationTarget->InhibitRedraw(true);

	operationTarget->GetSelection(&start, &end);

	operationTarget->SetSelection(aRange->start, aRange->end);
	operationTarget->GetTextRange(*aRange);

	operationTarget->SetSelection(start, end);
	operationTarget->InhibitRedraw(false);
	if (locked)
		operationTarget->UnlockWindowUpdate();
}

void CRETextAction::DeleteRange(const CTextRange *aRange)
{
	assert(aRange != 0);

	// select the range and deleted it
	operationTarget->SetSelection(aRange->start, aRange->end);
	operationTarget->PerformClear();
}

void CRETextAction::RestoreRange(const CTextRange *aRange)
{
	assert(aRange != 0);

	operationTarget->InhibitRedraw(true);

	operationTarget->InsertTextRange(*aRange);
	operationTarget->SetSelection(aRange->start, aRange->end);

	operationTarget->InhibitRedraw(false);
}

void CRETextAction::Finish()
{
	CREAction::Finish();

	long start;

	// save insertion endpoint
	operationTarget->GetSelection(&start, &(inserted->end));
}
