// RETextAction.h: interface for the CRETextAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// This class serves as a base class for all text actions
// which must save and restore a range of text (and text-like
// things). Typing actions like backspacing and editing actions
// like pasting are good examples.
//
// It defines the following operations for that purpose:
//
//		void SaveRange(CTextRange &range);
//			-- records all data, including objects, in the range
//			-- dictated by range.start and range.end
//
//		void RestoreRange(CTextRange &range);
//			-- restores the data by inserting it at range.start
//
//		void Undo();
//			-- saves inserted range if this is the first undo,
//			-- deletes the inserted range, then restores the
//			-- deleted range which was saved this object's ctor.
//
//		void Redo();
//			-- re-deletes the deleted range and then restores the
//			-- inserted range which was saved during Undo()
//
//////////////////////////////////////////////////////////////////////

#if !defined(RETEXTACTION_H)
#define RETEXTACTION_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "REAction.h"
#include "TextActionUtil.h"

class CRicherEditView;

class CRETextAction : public CREAction
{
public:
					CRETextAction(CRicherEditView *aView, UINT actionType);
	virtual			~CRETextAction();

	virtual	void	Undo();
	virtual	void	Redo();

	virtual	void	Finish();

protected:
					// info about what has been inserted
	CTextRange		*inserted;

					// info about what has been deleted
	CTextRange		*deleted;

					// indicates whether this action has ever
					// been undone; simplifies storing state.
	bool			neverUndone;

	void			SaveRange(CTextRange *aRange);
	void			DeleteRange(const CTextRange *aRange);
	void			RestoreRange(const CTextRange *aRange);
};

#endif

