// RETypingAction.cpp: implementation of the CRETypingAction class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RETypingAction.h"
#include "RicherEditView.h"

#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRETypingAction::CRETypingAction(CRicherEditView *aView, UINT actionType, void *)
	: CRETextAction(aView, actionType), firstChar(true), stillTyping(true)
{
}

CRETypingAction::~CRETypingAction()
{

}

void CRETypingAction::Do()
{
	// shouldn't happen
	assert(false);
}

void CRETypingAction::DoTyping(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// If it's the first char, make sure previous task was
	// finished. Otherwise, subsequent (normal) characters
	// are considered a part of this task, so don't call
	// CTask::Finish() inside of CTask::Do().
	if (firstChar)
	{
		firstChar = false;
		CREAction::Do();
	}

	if (nChar == VK_BACK)
		DoBackspace(nChar, nRepCnt, nFlags);
	// some chars change key codes (?) between WM_KEYDOWN
	// and WM_CHAR, e.g., '.' becomes VK_DELETE, while others
	// do not (e.g., VK_BACK is VK_BACK always??)
	else if (nChar == VK_DELETE && IsExtendedKey(nFlags))
		DoFwdDelete(nChar, nRepCnt, nFlags);
	else
		DoNormalChar(nChar, nRepCnt, nFlags);


}

void CRETypingAction::DoNormalChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{

	if (!operationTarget->InInsertMode())
	{
		RecordAfterCurrent();
	}

	operationTarget->PerformTyping(nChar, nRepCnt, nFlags);

	// adjust inserted range (we just inserted a new character)
	long start;
	operationTarget->GetSelection(&start, &(inserted->end));
	if (inserted->end < 0) inserted->end = 0;
	if (inserted->end < inserted->start) inserted->end = inserted->start;
}

void CRETypingAction::DoBackspace(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	long start, end;

	operationTarget->GetSelection(&start, &end);

	if (start > inserted->start)
	{
		// deleting text which is a part of this action
	}
	else if (end > start)
	{
		// deleting text which is currently selected;
		// that text is already saved so do nothing
	}
	else 							
	{
		// deleting text which is not a part of this
		// action, so we must record it and adjust the
		// previously recorded state appropriately

		CTextRange &textElement = operationTarget->ReserveUtilityRange();

		operationTarget->GetElementBefore(start, textElement);

 		// is there something before the cursor?
		if (textElement.is_empty() == false)
		{
			// yep, remove it from the inserted characters
			// and inserted at the beginning of what has
			// been deleted; because end of lines can have
			// multiple characters, we must use the size of
			// the item, not just 1.

			inserted->start -= textElement.size();
			deleted->prepend(textElement);
		}

		operationTarget->ReleaseUtilityRange();
	}

	operationTarget->PerformTyping(nChar, nRepCnt, nFlags);

	// adjust inserted range (we just deleted an inserted char)
	operationTarget->GetSelection(&start, &(inserted->end));
	if (inserted->end < 0) inserted->end = 0;
	if (inserted->end < inserted->start) inserted->end = inserted->start;
}

void CRETypingAction::DoFwdDelete(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	RecordAfterCurrent();
	operationTarget->PerformTyping(nChar, nRepCnt, nFlags);
}

void CRETypingAction::RecordAfterCurrent()
{
	long start, end;

	operationTarget->GetSelection(&start, &end);

	if (end > start)
	{
		// deleting text which is currently selected;
		// that text is already saved so do nothing
	}
	else
	{
		// deleting text which is not a part of this
		// action, so we must record it and adjust the
		// previously recorded state appropriately
		CTextRange &textElement = operationTarget->ReserveUtilityRange();

		operationTarget->GetElementAfter(start, textElement);

 		// is there something after the cursor?
		if (textElement.is_empty() == false)
		{

			// yep, append it to what's already been deleted
			// (can't affect inserted text as this only handles
			// left-to-right text)
			deleted->append(textElement);
		}
		operationTarget->ReleaseUtilityRange();
	}

	// forward delete does not affect the inserted range at all 
}

// indicates whether typing can continue; if false, a new
// typing action must be created for subsequent characters,
// making the previously inserted characters "distinct"
bool CRETypingAction::CanStillType() const
{
	return stillTyping;
}

// once this action finishes, we can't type anymore
void CRETypingAction::Finish()
{
	if (stillTyping)
		stillTyping = false;

	// Bypass saving the selection pts;
	// endpoints are already up to date
	CREAction::Finish();
}
