// RicherEditView.cpp : implementation file
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RicherEditView.h"
#include "UMResources.h"
#include "UndoManager.h"
#include "SimpleUndoRedoMenu.h"
#include "REAction.h"
#include "REEditAction.h"
#include "RETypingAction.h"
#include "REStyleAction.h"
#include "REParagraphAction.h"
#include "REReplaceAction.h"
#include "REInsertObjectAction.h"
#include "PasteSpecialData.h"

#include "MultipleUndoRedoMenu.h"

#include <afxodlgs.h>
#include <assert.h>

#if ENABLE_DRAG_DROP
#include "REDropAction.h"
#endif

// #include <afxrich.h>
// #include <RICHEDIT.H>
// #include "C:\Progra~1\DevStu~1\VC\MFC\src\viewrich.cpp"
// #include "C:\Progra~1\DevStu~1\VC\MFC\src\winctrl4.cpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CRicherEditView

IMPLEMENT_DYNCREATE(CRicherEditView, CRichEditView)

CLIPFORMAT CRicherEditView::m_cfRTF					= RegisterClipboardFormat(CF_RTF);

#ifdef ENABLE_DRAG_DROP
extern CLIPFORMAT cfRTF;

CLIPFORMAT CRicherEditView::m_cfPrivate				= RegisterClipboardFormat(CF_RETEXTOBJ);
CLIPFORMAT CRicherEditView::m_cfEmbeddedObject		= RegisterClipboardFormat( _T("Embedded Object") );
CLIPFORMAT CRicherEditView::m_cfObjectDescriptor	= RegisterClipboardFormat( _T("Object Descriptor") );
#endif

CFindReplaceDialog *CRicherEditView::findReplaceDialog = 0;
static const UINT nMsgFindReplace = ::RegisterWindowMessage(FINDMSGSTRING);

BEGIN_MESSAGE_MAP(CRicherEditView, CRichEditView)
	//{{AFX_MSG_MAP(CRicherEditView)
	ON_WM_CREATE()
	ON_COMMAND(ID_EDIT_CLEAR, OnEditClear)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_WM_CHAR()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_CHAR_BOLD, OnCharBold)
	ON_COMMAND(ID_CHAR_ITALIC, OnCharItalic)
	ON_COMMAND(ID_CHAR_UNDERLINE, OnCharUnderline)
	ON_COMMAND(ID_PARA_CENTER, OnParaCenter)
	ON_COMMAND(ID_PARA_LEFT, OnParaLeft)
	ON_COMMAND(ID_PARA_RIGHT, OnParaRight)
	ON_COMMAND(ID_FORMAT_FONT, OnFormatFont)
	ON_COMMAND(ID_OLE_INSERT_NEW, OnOleInsertNew)
	ON_COMMAND(ID_EDIT_PASTE_SPECIAL, OnEditPasteSpecial)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_UNDO_0, ID_UNDO_9, OnMultipleUndo)
	ON_UPDATE_COMMAND_UI_RANGE(ID_UNDO_0, ID_UNDO_9, OnUpdateMultipleUndo)
	ON_COMMAND_RANGE(ID_REDO_0, ID_REDO_9, OnMultipleRedo)
	ON_UPDATE_COMMAND_UI_RANGE(ID_REDO_0, ID_REDO_9, OnUpdateMultipleRedo)
	ON_REGISTERED_MESSAGE(nMsgFindReplace, OnFindReplaceCmd)
END_MESSAGE_MAP()


CRicherEditView::CRicherEditView()
	: lastAction(0), actionTarget(0), publicRange(0), privateRange(0),
	  publicRangeAvailable(true), privateRangeAvailable(true),
	  redraw(true), insertMode(true), lastChar(VK_NONAME)
{
}

CRicherEditView::~CRicherEditView()
{
	delete publicRange;
	delete privateRange;
}

int CRicherEditView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CRichEditView::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (GetActionTarget() == 0)
		SetActionTarget(GetDocument());

	// disable drag & drop
	CRichEditCtrl &reCtrl = GetRichEditCtrl();
	RevokeDragDrop(reCtrl.m_hWnd);

	// call factory method to create;
	publicRange = MakeTextRange();
	privateRange = MakeTextRange();

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CRicherEditView diagnostics

#ifdef _DEBUG
void CRicherEditView::AssertValid() const
{
	CRichEditView::AssertValid();
}

void CRicherEditView::Dump(CDumpContext& dc) const
{
	CRichEditView::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRicherEditView message handlers

// The four members eliminate the need for a deriving a new document class
// just to handle undo/redo. Note the document object is still used to record
// actions with the Undo Manager, as a single document may have several views,
// each just a "window" on the real data. This is also typical of how the
// MFC Library already handles undo/redo. Unfortunately, you will need to
// derive a subclass if the document is not the correct action target.

void CRicherEditView::OnEditUndo() 
{
	// tell undo manager to undo the last action
	// associate with the document (action target)
	gUndoManager->Undo(GetActionTarget());
}

void CRicherEditView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
//	was: CSimpleUndoRedoMenu::Set(pCmdUI, GetActionTarget());
	if (gUndoManager->CanUndo(GetActionTarget()))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CRicherEditView::OnEditRedo() 
{
	gUndoManager->Redo(GetActionTarget());
}

void CRicherEditView::OnUpdateEditRedo(CCmdUI* pCmdUI) 
{
//	was: CSimpleUndoRedoMenu::Set(pCmdUI, GetActionTarget());
	if (gUndoManager->CanRedo(GetActionTarget()))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);

}

// New handlers for multiple, simultaneous undo/redo
void CRicherEditView::OnMultipleUndo(UINT nID) 
{
	const long nActions = nID - ID_UNDO_0 + 1L;
	gUndoManager->Undo(GetActionTarget(), nActions);
}

void CRicherEditView::OnUpdateMultipleUndo(CCmdUI* pCmdUI) 
{
	const long nActions = pCmdUI->m_nID - ID_UNDO_0 + 1L;
	if (gUndoManager->CanUndo(GetActionTarget(), nActions))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CRicherEditView::OnMultipleRedo(UINT nID) 
{
	const long nActions = nID - ID_REDO_0 + 1L;
	gUndoManager->Redo(GetActionTarget(), nActions);
}

void CRicherEditView::OnUpdateMultipleRedo(CCmdUI* pCmdUI) 
{
	const long nActions = pCmdUI->m_nID - ID_REDO_0 + 1L;
	if (gUndoManager->CanRedo(GetActionTarget(), nActions))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

// Start of the standard message map overrides. Most of these following the
// following pattern, where XXX is the operation name:

//		void CRicherEditView::PerformXXX()
//				-- calls base handler, usually CRichEditView::OnXXX()

//		void CRicherEditView::OnXXX()
//				-- creates action object, then invokes Do(); action
//				-- objects eventually calls XXXOperation() after recording
//				-- appropriate state of view/document object

void CRicherEditView::PerformClear()
{
	CRichEditView::OnEditClear();
}

void CRicherEditView::OnEditClear() 
{
	lastAction = MakeEditAction(IDS_CLEAR_ACTION);
	lastAction->Do();
}

void CRicherEditView::PerformCut() 
{
	CRichEditView::OnEditCut();
}

void CRicherEditView::OnEditCut() 
{
	lastAction = MakeEditAction(IDS_CUT_ACTION);
	lastAction->Do();	
}


void CRicherEditView::PerformPaste() 
{
	CRichEditView::OnEditPaste();
}

void CRicherEditView::OnEditPaste() 
{
	lastAction = MakeEditAction(IDS_PASTE_ACTION);
	lastAction->Do();
}

void CRicherEditView::PerformPasteSpecial(CPasteSpecialData &psd)
{
	// Unlike the above PerformXXX()'s, we need to do this directly
	// as the base class throws up a paste special dialog box. We
	// have already done that below.

	GetRichEditCtrl().PasteSpecial(psd.cf, psd.dv, psd.hmf);
}

void CRicherEditView::OnEditPasteSpecial()
{
	COlePasteSpecialDialog dlg;
	dlg.AddStandardFormats();

	//	_oleData not defined (or could not find?), so use what it's equal to
	//	dlg.AddFormat(_oleData.cfRichTextFormat, TYMED_HGLOBAL, AFX_IDS_RTF_FORMAT, FALSE, FALSE);

	dlg.AddFormat(m_cfRTF, TYMED_HGLOBAL, AFX_IDS_RTF_FORMAT, FALSE, FALSE);
	dlg.AddFormat(CF_TEXT, TYMED_HGLOBAL, AFX_IDS_TEXT_FORMAT, FALSE, FALSE);

	if (dlg.DoModal() != IDOK)
		return;

	CPasteSpecialData pasteSpecialData	(
											dlg.GetDrawAspect(),
											(HMETAFILE)dlg.GetIconicMetafile(),
											dlg.m_ps.arrPasteEntries[dlg.m_ps.nSelectedIndex].fmtetc.cfFormat
										);

	CWaitCursor wait;
	SetCapture();

	// we set the target type so that QueryAcceptData know what to paste
	m_nPasteType = dlg.GetSelectionType();

	lastAction = MakeEditAction(IDS_PASTESPECIAL_ACTION, &pasteSpecialData);
	lastAction->Do();

	m_nPasteType = 0;

	ReleaseCapture();
}

// mouse clicks ends a typing action
void CRicherEditView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SelectionChanged();
	CRichEditView::OnLButtonDown(nFlags, point);
}

void CRicherEditView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Different characters are handled with different messages,
	// e.g., delete occurs during WM_KEYDOWN while normals chars
	// are handled during WM_CHAR.

	// Worse, the Edit & Rich Edit controls seem to differ in how
	// they handle chars too. E.g., backspace is handled during a
	// WM_CHAR in Edit control whereas it's handled during WM_KEYDOWN
	// in the Rich Edit control.

	// Through testing, I believe this routine handles all normal
	// characters, ie, no backspace delete, arrows, etc.

	// normal characters (VK_BACK not extended?)
	if (!IsExtendedKey(nFlags) && nChar != VK_BACK)
	{
		// Do we create a new action or can we reuse the last one?
		// We create a new action when

		//		* there wasn't a last action, typing or otherwise
		//		* or for some reason, the last action can't accommodate
		//		  more typing (may not even be a typing action)

		if (lastAction == 0 || lastAction->CanStillType() == false)
		{
			lastAction = MakeTypingAction(IDS_TYPING_ACTION);
		}

		// finally do the typing
		lastAction->DoTyping(nChar, nRepCnt, nFlags);
	}
	else
		// either an action was created below or the char doesn't
		// need an action (it may be something like shift or ctrl)
		CRichEditView::OnChar(nChar, nRepCnt, nFlags);
}

void CRicherEditView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Different characters are handled with different messages,
	// e.g., delete occurs during WM_KEYDOWN while normals chars
	// are handled during WM_CHAR.

	// Worse, the Edit & Rich Edit controls seem to differ in how
	// they handle chars too. E.g., backspace is handled during a
	// WM_CHAR in Edit control whereas it's handled during WM_KEYDOWN
	// in the Rich Edit control.

	// Through testing, I believe this routine handles all editing
	// characters, like backspace and delete, plus cursor keys.

	if (nChar == VK_INSERT)
	{
		insertMode = !insertMode;
		CRichEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	}
	else if (IsEditingKey(nChar))
	{
		long start, end;

		GetSelection(&start, &end);

		// We don't need an action if we have a single
		// selection point and we are either 

		//		* backspacing at the beginning of the text
		//		* forward deleting at the end of the text

		// because there's nothing to undo

		if (start == end)
		{
			if	(	(nChar == VK_BACK && start == 0)
				||
					(nChar == VK_DELETE && start == GetLength()))
			{
				CRichEditView::OnKeyDown(nChar, nRepCnt, nFlags);
				return;
			}
		}

		// Okay, there's something that will be deleted; now see 
		// if we should create a new action or whether we can re-
		// use the last one. We do this when

		//		* there wasn't a last action, typing or otherwise
		//		* we've just started to backspace over previous text
		//		* or for some reason, the last action can't accommodate
		//		  more typing (may not even be a typing action)

		if (
				lastAction == 0
			||
				(nChar == VK_BACK && lastChar != VK_BACK)
			||
				lastAction->CanStillType() == false
			)
		{
			lastAction = MakeTypingAction(IDS_TYPING_ACTION);
		}

		// finally do the typing
		lastAction->DoTyping(nChar, nRepCnt, nFlags);
	}
	else if (IsCursorKey(nChar))
	{
		// cursor keys don't affect the text, just the cursor
		SelectionChanged();
		CRichEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	}
	else
		// the character is handled in OnChar()
		CRichEditView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CRicherEditView::PerformTyping(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// This routine is the bottleneck for typing, ie, after this
	// routine executes, a new character has been applied to the
	// buffer. Note that a delete or backspace can actually
	// remove characters from the it, so it does not always add.

	// Because different keys are handled at different times, the
	// base member which is called depends on the character.

	if (IsExtendedKey(nFlags) || nChar == VK_BACK)
	{
		CRichEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	}
	else
	{
		CRichEditView::OnChar(nChar, nRepCnt, nFlags);
	}

	lastChar = nChar;
}

void CRicherEditView::OnStyle(DWORD dwMask, DWORD dwEffect)
{
	OnStyle(dwMask, dwEffect, IDS_STYLECHANGE_ACTION);
}

void CRicherEditView::OnStyle(DWORD dwMask, DWORD dwEffect, UINT actionType)
{
	StyleActionData data = { dwMask, dwEffect };
	lastAction = MakeStyleAction(actionType, &data);
	lastAction->Do();
}

void CRicherEditView::PerformStyle(DWORD dwMask, DWORD dwEffect)
{
	CRichEditView::OnCharEffect(dwMask, dwEffect);
}

void CRicherEditView::OnCharBold() 
{
	OnStyle(CFM_BOLD, CFE_BOLD, IDS_BOLD_ACTION);	
}

void CRicherEditView::OnCharItalic() 
{
	OnStyle(CFM_ITALIC, CFE_ITALIC, IDS_ITALICS_ACTION);
}

void CRicherEditView::OnCharUnderline() 
{
	OnStyle(CFM_UNDERLINE, CFE_UNDERLINE, IDS_UNDERLINE_ACTION);	
}

void CRicherEditView::OnColorPick(UINT nID)
{
	lastAction = MakeStyleAction(IDS_COLORCHANGE_ACTION, &nID);
	lastAction->Do();
}

void CRicherEditView::PerformColorPick(UINT nID)
{
	CRichEditView::OnColorPick(nID);
}

void CRicherEditView::OnColorDefault()
{
	lastAction = MakeStyleAction(IDS_DEFAULTCOLOR_ACTION);
	lastAction->Do();
}

void CRicherEditView::PerformColorDefault()
{
	CRichEditView::OnColorDefault();
}

void CRicherEditView::OnParaCenter() 
{
	OnParaAlign(PFA_CENTER, IDS_CENTERJUSTIFICATION_ACTION);
}

void CRicherEditView::OnParaLeft() 
{
	OnParaAlign(PFA_LEFT, IDS_LEFTJUSTIFICATION_ACTION);	
}

void CRicherEditView::OnParaRight() 
{
	OnParaAlign(PFA_RIGHT, IDS_RIGHTJUSTIFICATION_ACTION);		
}

void CRicherEditView::OnParaAlign(WORD alignment)
{
	OnParaAlign(alignment, IDS_JUSTIFICATION_ACTION);
}

void CRicherEditView::OnParaAlign(WORD alignment, UINT actionType)
{
	lastAction = MakeParagraphAction(actionType, &alignment);
	lastAction->Do();
}

void CRicherEditView::PerformParaAlign(WORD alignment)
{
	CRichEditView::OnParaAlign(alignment);
}

void CRicherEditView::OnSetParaFormat(PARAFORMAT &pf)
{
	lastAction = MakeParagraphAction(IDS_PARAGRAPHFORMATTING_ACTION, &pf);
	lastAction->Do();
}

void CRicherEditView::OnBullet()
{
	lastAction = MakeParagraphAction(IDS_BULLETS_ACTION);
	lastAction->Do();
}

void CRicherEditView::PerformBullet()
{
	CRichEditView::OnBullet();
}

void CRicherEditView::OnSetCharFormat(const CHARFORMAT &cf)
{
	UINT actionType = IDS_FONTFORMATTING_ACTION;

	if (cf.dwMask == CFM_SIZE)
		actionType = IDS_SIZECHANGE_ACTION;

	lastAction = MakeStyleAction(actionType, (void *)&cf);
	lastAction->Do();
}

void CRicherEditView::OnFormatFont()
{
	GetCharFormatSelection();
	CFontDialog dlg(m_charformat, CF_BOTH|CF_NOOEMFONTS);
	if (dlg.DoModal() == IDOK)
	{
		dlg.GetCharFormat(m_charformat);
		OnSetCharFormat(m_charformat);
	}
}

void CRicherEditView::OnOleInsertNew() 
{
	// Invoke the standard Insert Object dialog box to obtain information
	COleInsertDialog dlg;
	if (dlg.DoModal() != IDOK)
		return;

	CREAction *anAction = MakeInsertObjectAction(IDS_INSERTOBJECT_ACTION, &dlg);

	anAction->Do();
	if (anAction->IsSuccess())
	{
		anAction->NotifyUndoManager();
		lastAction = anAction;
	}
	else
		delete anAction;
}

bool CRicherEditView::PerformInsertObject(COleInsertDialog &dlg)
{
	bool success = true;
	CWaitCursor wait;

	CRichEditCntrItem* pItem = NULL;
	TRY
	{
		// create item from dialog results
		pItem = GetDocument()->CreateClientItem();
		pItem->m_bLock = TRUE;
		if (!dlg.CreateItem(pItem))
		{
			pItem->m_bLock = FALSE;
			AfxThrowMemoryException();  // any exception will do
		}

		HRESULT hr = InsertItem(pItem);
		pItem->UpdateItemType();

		pItem->m_bLock = FALSE;

		if (hr != NOERROR)
			AfxThrowOleException(hr);

		// if insert new object -- initially show the object
		if (dlg.GetSelectionType() == COleInsertDialog::createNewItem)
			pItem->DoVerb(OLEIVERB_SHOW, this);
	}
	CATCH(CException, e)
	{
		success = false;
		if (pItem != NULL)
		{
			ASSERT_VALID(pItem);
			pItem->Delete();
		}
		AfxMessageBox(AFX_IDP_FAILED_TO_CREATE);
	}
	END_CATCH

	return success;
}

LRESULT CRicherEditView::OnFindReplaceCmd(WPARAM wParam, LPARAM lParam)
{
	// We can only hope this is the right one, as the edit state
	// classes are "private" to MFC (in afximpl.h). Seems to work,
	// but there's really no documentation I could find.

	findReplaceDialog = CFindReplaceDialog::GetNotifier(lParam);
	ASSERT(findReplaceDialog != NULL);

	return CRichEditView::OnFindReplaceCmd(wParam, lParam);
}

void CRicherEditView::OnReplaceSel(LPCTSTR lpszFind, BOOL bNext, BOOL bCase,
	BOOL bWord, LPCTSTR lpszReplace)
{
	// Ditto for these next two routines (OnReplaceSel() and OnReplaceAll()).
	// As some things are private, like the edit state, we cannot hope to
	// use pEditState->strFind or the other member variabs. Again, it seems
	// to work...

	ASSERT_VALID(this);
	ASSERT(findReplaceDialog != NULL);

	if (!SameAsSelected(lpszFind, bCase, bWord))
	{
		if (!FindText(lpszFind, bCase, bWord))
			TextNotFound(lpszFind);
		else
			AdjustDialogPosition(findReplaceDialog);
		return;
	}

	ReplaceData data(lpszReplace);

	lastAction = MakeReplaceAction(IDS_REPLACE_ACTION, &data);
	lastAction->Do();

	if (!FindText(lpszFind, bCase, bWord))
		TextNotFound(lpszFind);
	else
		AdjustDialogPosition(findReplaceDialog);
	ASSERT_VALID(this);
}

void CRicherEditView::OnReplaceAll(LPCTSTR lpszFind, LPCTSTR lpszReplace, BOOL bCase, BOOL bWord)
{
	ASSERT_VALID(this);
	ASSERT(findReplaceDialog != NULL);

	CWaitCursor wait;
	// no selection or different than what looking for
	if (!SameAsSelected(lpszFind, bCase, bWord))
	{
		if (!FindText(lpszFind, bCase, bWord))
		{
			TextNotFound(lpszFind);
			return;
		}
	}

	ReplaceData data(lpszReplace, lpszFind, bCase, bWord);

	GetRichEditCtrl().HideSelection(TRUE, FALSE);
	lastAction = MakeReplaceAction(IDS_REPLACEALL_ACTION, &data);
	lastAction->Do();
	GetRichEditCtrl().HideSelection(FALSE, FALSE);

	ASSERT_VALID(this);
}

// factory methods for creating actions
CREAction *CRicherEditView::MakeEditAction(UINT actionType, void *data)
{
	return new CREEditAction(this, actionType, data);
}

CREAction *CRicherEditView::MakeTypingAction(UINT actionType, void *data)
{
	return new CRETypingAction(this, actionType, data);
}

CREAction *CRicherEditView::MakeStyleAction(UINT actionType, void *data)
{
	return new CREStyleAction(this, actionType, data);
}

CREAction *CRicherEditView::MakeParagraphAction(UINT actionType, void *data)
{
	return new CREParagraphAction(this, actionType, data);
}

CREAction *CRicherEditView::MakeReplaceAction(UINT actionType, void *data)
{
	return new CREReplaceAction(this, actionType, data);
}

CREAction *CRicherEditView::MakeInsertObjectAction(UINT actionType, void *data)
{
	return new CREInsertObjectAction(this, actionType, data);
}

// factory method for creating a text range; subclasses which extend a
// range with additional attributes should override this member function
// to create the appropriate CTextRange subclass; used to create the
// utility ranges and within most r.e.-actions
CTextRange *CRicherEditView::MakeTextRange()
{
	return new CTextRange();
}

// this routine cancels the current typing action if any. You can call
// this whenever you want to create a distinct typing action which can
// be undone. Unlike EN_SELCHANGE, this allows more control over when
// typing and other actions are actually finished. For example, typing
// some text then backspacing is considered a single action. 
void CRicherEditView::SelectionChanged()
{

	if (lastAction)
	{
		if (lastAction->IsFinished() == false)
			lastAction->Finish();
		lastAction = 0;
	}
}

// supports typing actions, backspacing in this case
void CRicherEditView::GetElementBefore(long pos, CTextRange &aRange)
{
	assert(0 <= pos && pos <= GetLength());

	if (pos != 0)
	{
		long selStart, selEnd;

		HideSelection(TRUE, FALSE);

		GetSelection(&selStart, &selEnd);
		SetSelection(pos - 1, pos);

		// test for funny end of line
		long testStart, testEnd;
		GetSelection(&testStart, &testEnd);
		if (testStart == testEnd)
		{
			SetSelection(pos - 2, pos);
		}

		GetTextRange(aRange);
		SetSelection(selStart, selEnd);
		HideSelection(FALSE, TRUE);
	}
	else
	{
		aRange.end = aRange.start = 0;
	}
}

// supports typing actions, in this case forward delete
void CRicherEditView::GetElementAfter(long pos, CTextRange &aRange)
{
	const long len = GetLength();
	assert(0 <= pos && pos <= len);

	if (pos < len)
	{
		long selStart, selEnd;

		HideSelection(TRUE, FALSE);

		GetSelection(&selStart, &selEnd);
		SetSelection(pos, pos + 1);

		// test for funny end of line
		long testStart, testEnd;
		GetSelection(&testStart, &testEnd);
		if (testStart == testEnd)
		{
			SetSelection(pos, pos + 2);
		}

		GetTextRange(aRange);
		SetSelection(selStart, selEnd);

		HideSelection(FALSE, FALSE);
	}
	else
	{
		aRange.end = aRange.start = pos;
	}

}

///////////////////////////////////////////////////////////////

// gets the styles (font, size, bold, etc.) associated with the
// currently selected range; because the range is resized, any
// previous range data is destroyed.
void CRicherEditView::GetTextStyles(CTextRange &aRange) const
{
	long end;
	CRicherEditView * const that = const_cast<CRicherEditView * const>(this);

	aRange.resize(0);

	// get a "temporary" range to store the attributes
	// of an individual text element; resize it so that
	// there's space for an one piece of style data
	CTextRange &textElement = that->ReservePrivateRange();
	textElement.styleTable.resize(1);

	// aRange.end will be computed by append()
	GetSelection(&aRange.start, &end);
	aRange.end = aRange.start;

	// selection could be empty; no text styles then (maybe
	// store default style for those subclasses that needed?)
	if (aRange.start < end)
	{
		for(long i = aRange.start; i < end; ++i)
		{
			textElement.start = i;
			textElement.end = i+1;
			that->SetSelection(i, i+1);
			GetTextStyle(i, textElement);
			aRange.append(textElement);
		}
		
		that->SetSelection(aRange.start, aRange.end);
	}

	// don't forget to release the "temporary"
	that->ReleasePrivateRange();

}

// (non-overridable) bottleneck used to get the attributes associated
// with a single text element at position i; don't forget the run count!
void CRicherEditView::GetTextStyle(long, CTextRange &aRange) const
{
	CTextStyle &current = aRange.styleTable[0];
	GetRichEditCtrl().GetSelectionCharFormat(current.format);
	current.count = 1;
}

// sets the styles of the range specified in aRange; note the
// flag preserveSelection applies to the selection range that
// existed before the call.
void CRicherEditView::SetTextStyles(const CTextRange &aRange,
									bool preserveSelection)
{
	long start, end;
	CStyleTable::const_iterator ptr		= aRange.styleTable.begin();
	CStyleTable::const_iterator endPtr	= aRange.styleTable.end();

	if (preserveSelection)
		GetSelection(&start, &end);

	CRichEditCtrl &reCtrl = GetRichEditCtrl();
	long last = aRange.start;
	for(; ptr != endPtr; ++ptr)
	{
		const long next = last + (*ptr).count;

		SetSelection(last, next);
		reCtrl.SetSelectionCharFormat(const_cast<CHARFORMAT &>((*ptr).format));
		last = next;
	}

	if (preserveSelection)
		SetSelection(start, end);
	m_bSyncCharFormat = TRUE;
}


///////////////////////////////////////////////////////////////

// gets the paragraph styles (justification) associated with the
// currently selected range; because the range is resized, any
// previous range data is destroyed.
void CRicherEditView::GetParagraphStyles(CTextRange &aRange) const
{
	long end;
	CRicherEditView * const that = const_cast<CRicherEditView * const>(this);

	aRange.resize(0);

	CTextRange &textElement = that->ReservePrivateRange();
	textElement.paragraphTable.resize(1);

	// aRange.end will be computed by append()
	GetSelection(&aRange.start, &end);
	aRange.end = aRange.start;

	if (aRange.start < end)
	{
		for(long i = aRange.start; i < end; ++i)
		{
			textElement.start = i;
			textElement.end = i+1;
			that->SetSelection(i, i+1);
			GetParagraphStyle(i, textElement);
			aRange.append(textElement);
		}

	}
	// a paragraph style affects the entire paragraph, so even if
	// the selection is a single point, there's some data to save
	else if (aRange.start == end)
	{
		that->SetSelection(end, end);
		GetParagraphStyle(end, textElement);
		aRange.append(textElement);
	}

	that->ReleasePrivateRange();
	that->SetSelection(aRange.start, aRange.end);
}

// (non-overridable) bottleneck used to get the attributes associated
// with a single text element at position i; don't forget the run count!
void CRicherEditView::GetParagraphStyle(long, CTextRange &textElement) const
{
	assert(textElement.paragraphTable.size() == 1);
	CParagraphStyle &current = textElement.paragraphTable[0];

	GetRichEditCtrl().GetParaFormat(current.format);
	current.count = 1;
	current.SetMask();
}

// sets the paragraph styles of the range specified in aRange; note
// the flag preserveSelection applies to the selection range that
// existed before the call.
void CRicherEditView::SetParagraphStyles(const CTextRange &aRange,
										 bool preserveSelection)
{
	long start, end;
	CParagraphTable::const_iterator ptr		= aRange.paragraphTable.begin();
	CParagraphTable::const_iterator endPtr	= aRange.paragraphTable.end();

	if (preserveSelection)
		GetSelection(&start, &end);

	long last = aRange.start;
	for(; ptr != endPtr; ++ptr)
	{
		const long next = last + (*ptr).count;

		SetSelection(last, next);
		CRichEditView::SetParaFormat(const_cast<PARAFORMAT &>((*ptr).format));
		last = next;
	}

	if (preserveSelection)
		SetSelection(start, end);
	m_bSyncParaFormat = TRUE;
}

///////////////////////////////////////////////////////////////

// insert (OLE) objects at the positions specified in the ObjectData
// structures, rather than (necessarily) within the selection range.
void CRicherEditView::InsertObjects(const CTextRange &aRange,
									bool preserveSelection)
{
	long selStart, selEnd;

	if (preserveSelection)
		GetSelection(&selStart, &selEnd);
	
	CObjectTable::const_iterator ptr = aRange.objectTable.begin();
	CObjectTable::const_iterator end = aRange.objectTable.end();
	for(; ptr != end; ++ptr)
	{
		if ((*ptr).storage)
		{
			CMemFile storage((*ptr).storage, (*ptr).size);
			CArchive archive(&storage, CArchive::load);
			CRichEditCntrItem *storedItem = new CRichEditCntrItem(0, GetDocument());

			storedItem->Serialize(archive);

			archive.Close();
			storage.Close();

			SetSelection((*ptr).position, (*ptr).position);
			InsertItem(storedItem);
		}
	}

	if (preserveSelection)
		SetSelection(selStart, selEnd);

	m_bSyncCharFormat = m_bSyncParaFormat = TRUE;
}

// (non-overridable) bottleneck for getting OLE objects; does not
// use run counts or offsets, but rather absolute positions; I wouldn't
// rely on this however (!), and of course, positions outside the
// a range's endpoints aren't well-defined
void CRicherEditView::GetObject(long i, CTextRange &textElement) const
{
	if (GetRichEditCtrl().GetSelectionType() == SEL_OBJECT)
	{
		CRichEditCntrItem *selectedItem = GetSelectedItem();
		if (selectedItem)
		{
			CMemFile storage;
			CArchive archive(&storage, CArchive::store);

			selectedItem->Serialize(archive);
			archive.Close();

			textElement.objectTable.resize(1);
			CObjectData &objectData = textElement.objectTable[0];

			objectData.position = i;
			objectData.size = storage.GetLength();
			objectData.storage = storage.Detach();
		}
	}
	else
		textElement.objectTable.resize(0);
}

///////////////////////////////////////////////////////////////

// GetTextRange() is a template method that saves all attributes
// associated with the current selection; subclasses should override
// the bottleneck below.
void CRicherEditView::GetTextRange(CTextRange &aRange) const
{
	long end;
	CRicherEditView * const that = const_cast<CRicherEditView * const>(this);

	// data.end will be computed by append()
	GetSelection(&aRange.start, &end);

	aRange.end = aRange.start;
	aRange.text = GetTextSelection();

	if (aRange.start < end)
	{
		CTextRange &textElement = that->ReservePrivateRange();
		textElement.resize(1);

		for(long i = aRange.start; i < end; ++i)
		{
			textElement.start = i;
			textElement.end = i+1;
			that->SetSelection(i, i + 1);
			GetTextRange(i, textElement);
			aRange.append(textElement);
		}

		that->ReleasePrivateRange();
		that->SetSelection(aRange.start, aRange.end);
	}
}


// This routine forms the main range saving bottleneck. The caller
// expects this function to get the range information at position i;
// subclasses refine this routine to add new kinds of range attributes,
// e.g., an annotation "style". The other Get() functions provide good
// examples of how to do this easily, so that only one or two routines
// actually have nontrivial definitons.

// "Walking" through the text is a very slow operation, even if redrawing
// is off. For that reason, we only want to do it once, hence the use of
// this bottleneck. Because the attibutes are usually sparse, e.g., the
// run count for each record is large, the Set() routines are less affected.

void CRicherEditView::GetTextRange(long i, CTextRange &data) const
{
	GetObject(i, data);
	GetTextStyle(i, data);
	GetParagraphStyle(i, data);
}

// Inserts a full range of information. Actually, this routine can also be
// used to restore individual range attributes, like the text style, as long
// as the other attributes are empty, though using a specific function like
// SetTextStyles() seems easier to understand. Override this to handle new
// attributes. Note the the flag preseveSelection applies to the selection
// which existed before the call.
void CRicherEditView::InsertTextRange(const CTextRange &aRange, bool preserveSelection)
{
	long start, end;

	if (preserveSelection)
		GetSelection(&start, &end);

	SetSelection(aRange.start, aRange.start);
	InsertText(aRange.text);
	InsertObjects(aRange, false);
	SetTextStyles(aRange, false);
	SetParagraphStyles(aRange, false);

	if (preserveSelection)
		SetSelection(start, end);
}

// Turn off redrawing, and anything related to it,
// to avoid the flashing inherent in selecting and
// reselection ranges of text. While you can nest
// calls this, the first call with disable == false
// turns drawing back on.
void CRicherEditView::InhibitRedraw(const bool disable)
{
	if (disable)
	{
		if (redraw)
		{
			redraw = false;
			locked = LockWindowUpdate();
			redrawMask = GetRichEditCtrl().GetEventMask();
			GetRichEditCtrl().SetEventMask(ENM_NONE);
			HideSelection(TRUE, FALSE);
		}
	}
	else
	{
		if (redraw == false)
		{
			redraw = true;
			if (locked)
			{
				locked = false;
				UnlockWindowUpdate();
			}
			GetRichEditCtrl().SetEventMask(redrawMask);
			HideSelection(FALSE, FALSE);
		}
	}
}

// These two routines are useful for hit-testing and thus, drag & drop,
// As I indicated in the guide, drag & drop must be fully re-implemented
// as far as I can tell, and this is part of what needs to be done.

// Because drap & drop isn't supported yet, these routines have not been
// as tested nor tuned as the others, but they seem to work for me.

// Both use a binary search, first over lines, then once a line is found,
// over the characters in the line. This should be adequate for most usage,
// as long & wide documents are not really feasible with the rich edit ctrl,
// though a simple optimization is to do cache the last hit and do nearby
// testing first.

long CRicherEditView::GetLineIndexFromPoint(const CPoint &aPoint) const
{
	const CRichEditCtrl &reCtrl = GetRichEditCtrl();
	const long lastLine = reCtrl.GetLineCount() - 1;
	long startLine = 0, endLine = reCtrl.LineFromChar(GetTextLength());

	while (startLine < endLine)
	{
		const long midLine = (startLine + endLine) / 2;
		const long midChar = reCtrl.LineIndex(midLine);
		const CPoint midPoint = reCtrl.GetCharPos(midChar);

		if (midPoint.y < aPoint.y)
		{
			if (midLine < lastLine)
			{
				const long nextChar = reCtrl.LineIndex(midLine + 1);
				const CPoint nextPoint = reCtrl.GetCharPos(nextChar);

				if (nextPoint.y - 1 < aPoint.y)
					startLine = midLine + 1;
				else
					startLine = endLine = midLine;
			}
			else
				startLine = endLine = midLine;
		}
		else if (midPoint.y > aPoint.y)
		{
				endLine = midLine - 1;
		}
		else
		{
			startLine = endLine = midLine;
		}
	}

	return endLine;
}

long CRicherEditView::GetCharIndexFromPoint(const CPoint &aPoint) const
{
	const CRichEditCtrl &reCtrl = GetRichEditCtrl();

	const long startLine = GetLineIndexFromPoint(aPoint);
	long startChar = reCtrl.LineIndex(startLine), endChar = startChar + reCtrl.LineLength(startLine) - 1;
	const long lastChar = endChar;

	while (startChar < endChar)
	{
		const long midChar = (startChar + endChar) / 2;
		const CPoint midPoint = reCtrl.GetCharPos(midChar);
		
		if (midPoint.x < aPoint.x)
		{
			if (midChar < lastChar)
			{
				const CPoint nextPoint = reCtrl.GetCharPos(midChar + 1);
				if (nextPoint.x < aPoint.x)
					startChar = midChar + 1;
				else
					startChar = endChar = midChar;
			}
			else
				startChar = endChar = midChar;
		}
		else if (midPoint.x > aPoint.x)
		{
			endChar = midChar - 1;
		}
		else
		{
			startChar = endChar;
		}
	}

	return startChar;
}

// Drag & drop is not supported yet. If the MFC Library used Rich Edit
// 2.0, this would be easy, as there's a notification message sent when
// a drop is done. With 1.0 however, you essentially have to do everything
// your self, which turns out to be awful lot! What's worse is that most
// of the routines needed are either undocumented, or hidden in the rich
// edit control, or both.

// If you want to use/fix the routines below, note that some of them
// are (Class Wizard) overrides and should be listed in the AFX_VIRTUAL
// section of the header, e.g., OnDrop(). Others are listed but are
// controlled by the ENABLE_DRAG_DROP flag.

// Also note that DragEnter(), DragOver(), and few others would need to
// be defined. My versions eventually became so convoluted I deleted
// them in disgust.

#if ENABLE_DRAG_DROP
bool CRicherEditView::CanUseDataObject(COleDataObject* pDataObject)
{
	return 
		(
				pDataObject->IsDataAvailable(CF_TEXT)
			||
				pDataObject->IsDataAvailable( m_cfRTF )
			||
				pDataObject->IsDataAvailable( m_cfPrivate )
			||
				pDataObject->IsDataAvailable( m_cfObjectDescriptor )
			||
				pDataObject->IsDataAvailable(CF_METAFILEPICT)
			||
				pDataObject->IsDataAvailable(CF_DIB)
			||
				pDataObject->IsDataAvailable(CF_BITMAP)
		) == TRUE;
}

BOOL CRicherEditView::ImportFromDataObject(COleDataObject *dataObject)
{
#if 0
	IRichEditOle *reOle = GetRichEditCtrl().GetIRichEditOle();
	LPDATAOBJECT oleData = dataObject->GetIDataObject(TRUE);
	reOle->ImportDataObject(oleData, 0, 0);

	GetDocument()->UpdateAllItems(NULL);
	GetDocument()->UpdateAllViews(NULL);
	return TRUE;
#else
	CRichEditView::DoPaste(*dataObject, cfRTF, 0);
	return TRUE;
#endif
}

DROPEFFECT CRicherEditView::OnDropEx(COleDataObject* pDataObject, DROPEFFECT dropDefault, DROPEFFECT dropList, CPoint point) 
{
	if (GetKeyState(VK_RBUTTON) == 0)
		return CRichEditView::OnDropEx(pDataObject, dropDefault, dropList, point);
	else
	{
		// do popup
		return -1;
	}
}

BOOL CRicherEditView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) 
{
	if (dropEffect == DROPEFFECT_MOVE || dropEffect == DROPEFFECT_COPY)
	{
		const UINT actionType = (dropEffect == DROPEFFECT_MOVE) ? IDS_MOVE_ACTION : IDS_DUPLICATE_ACTION;
		long dropIndex = GetCharIndexFromPoint(point);
		
		lastAction = MakeDropAction(actionType, dropIndex, pDataObject, m_bInDrag);
		lastAction->Do();

		return !m_bInDrag;
	}

	return FALSE;
}

CREAction *CRicherEditView::MakeDropAction(UINT actionType, long dropIndex, COleDataObject *pDataObject, BOOL selfDrag)
{
	return new CREDropAction(this, actionType, dropIndex, pDataObject, selfDrag);
}

#endif

void CRicherEditView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	// force menu rebuild by creating "NULL" action
	// (eventually, the notification callback calls
	// the appropriate menu building routine)
	if (bActivate && IsWindowVisible())
		gUndoManager->SetLastAction(GetActionTarget(), 0);
	CRichEditView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}
