// SimpleUndoRedoMenu.cpp: implementation of the CSimpleUndoRedoMenu class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UMResources.h"
#include "UndoManager.h"
#include "SimpleUndoRedoMenu.h"

#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// while using global const CString's is not necessarily efficient, it's
// necessary to support UNICODE or other double-byte character systems
const CString CSimpleUndoRedoMenu::menuItem[2] = { _T("&Undo "), _T("&Redo ") };
const CString CSimpleUndoRedoMenu::accelerator[2] = { _T("\tCtrl+Z"), _T("\tCtrl+Y") };
const CString CSimpleUndoRedoMenu::undoItem( _T("&Undo") );
const CString CSimpleUndoRedoMenu::redoItem( _T("&Redo") );

void CSimpleUndoRedoMenu::Build(CMenu *menuBar, const void *actionTarget)
{
	CMenu *editMenu = 0;
	CString anItem;
	const int numMenus = menuBar->GetMenuItemCount();

	// assumes Undo/Redo are menu items and that
	// they are the first and second items listed
	for (int i = 0; i < numMenus; ++i)
	{
		CMenu *aMenu = menuBar->GetSubMenu(i);
		if (aMenu)
		{
			aMenu->GetMenuString(0, anItem, MF_BYPOSITION);
			if (anItem.Left(5) == undoItem)
			{
				editMenu = aMenu;
				break;
			}
		}
	}

	// it has to be there
	if (editMenu)
	{
		// update the undo item label
		{
			CString actionName;
			if (gUndoManager->CanUndo(actionTarget))
				actionName = gUndoManager->GetActionName(actionTarget, CUndoManager::UNDO, 0);

			CString caption = menuItem[0] + actionName + accelerator[0];
			editMenu->ModifyMenu(0, MF_BYPOSITION, ID_EDIT_UNDO, caption);
		}

		// update the redo item label
		{
			CString actionName;
			if (gUndoManager->CanRedo(actionTarget))
				actionName = gUndoManager->GetActionName(actionTarget, CUndoManager::REDO, 0);

			CString caption = menuItem[1] + actionName + accelerator[1];
			editMenu->ModifyMenu(1, MF_BYPOSITION, ID_EDIT_REDO, caption);
		}
	}
}

void UMSimpleMenuBuilder(void *actionTarget, UMNotificationData &data)
{
	// assumes the main wnd is associated with the action target
	// (not necessarily true but probably a reasonable assumption)
	CSimpleUndoRedoMenu::Build(AfxGetMainWnd()->GetMenu(), actionTarget);
	UMCmdTargetNotifier(actionTarget, data);
}
