// TextStyle.h: interface for the CTextStyle class.
//
// Copyright (c) 1998 Matt Clay, all rights reserved.
//
// Utility data structure for a single text style.
//
//////////////////////////////////////////////////////////////////////

#if !defined(TEXTSTYLE_H)
#define TEXTSTYLE_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <richedit.h>

class CTextStyle  
{
public:
	CTextStyle()
	{
		count = 0;

		format.cbSize = sizeof(CHARFORMAT);
		format.dwMask = 0;
		format.dwEffects = 0;
		format.yHeight = 9;
		format.yOffset = 0;
		format.crTextColor = 0;
		format.bCharSet = DEFAULT_CHARSET;
		format.bPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
		strcpy(format.szFaceName, "Fixedsys");
	}
	CTextStyle(const CHARFORMAT &cf)
		: count(1), format(cf)
	{ }
	CTextStyle(const CTextStyle &ts)
	{
		count = ts.count;

		format.cbSize = ts.format.cbSize;
		format.dwMask = ts.format.dwMask; 
		format.dwEffects = ts.format.dwEffects; 
		format.yHeight = ts.format.yHeight; 
		format.yOffset = ts.format.yOffset; 
		format.crTextColor = ts.format.crTextColor; 
		format.bCharSet = ts.format.bCharSet; 
		format.bPitchAndFamily = ts.format.bPitchAndFamily;

		strcpy(format.szFaceName, ts.format.szFaceName); 
	}

	const CTextStyle &operator=(const CTextStyle &ts)
	{
		if (this != &ts)
		{
			count = ts.count;

			format.cbSize = ts.format.cbSize;
			format.dwMask = ts.format.dwMask; 
			format.dwEffects = ts.format.dwEffects; 
			format.yHeight = ts.format.yHeight; 
			format.yOffset = ts.format.yOffset; 
			format.crTextColor = ts.format.crTextColor; 
			format.bCharSet = ts.format.bCharSet; 
			format.bPitchAndFamily = ts.format.bPitchAndFamily;

			strcpy(format.szFaceName, ts.format.szFaceName); 
		}

		return *this;
	}

	long count;
	CHARFORMAT format;
};

inline bool operator==(const CTextStyle &ts1, const CTextStyle &ts2)
{
	// ignore start and end

	return		ts1.format.cbSize == ts2.format.cbSize
			&&
				ts1.format.dwMask == ts2.format.dwMask
			&&
				ts1.format.dwEffects == ts2.format.dwEffects
			&&
				ts1.format.yHeight == ts2.format.yHeight
			&&
				ts1.format.yOffset == ts2.format.yOffset 
			&&
				ts1.format.crTextColor == ts2.format.crTextColor 
			&&
				ts1.format.bCharSet == ts2.format.bCharSet
			&&
				ts1.format.bPitchAndFamily == ts2.format.bPitchAndFamily
			&&
				(strcmp(ts1.format.szFaceName, ts2.format.szFaceName) == 0);
}

inline bool operator!=(const CTextStyle &ts1, const CTextStyle &ts2)
{
	return !(ts1 == ts2);
}

#endif
